<?php
// Exit if runs outside WP.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Lytex_pagamentos_API extends WC_lytex_Gateway {


    /**
     * Informa a URL correta da API
     * @param $metod
     * @return string
     */
    public function get_base_url($metod) {
        if($this->sandbox == "no"){
            return 'https://'.$metod.'-pay.lytex.com.br';
        }else{
            return 'https://sandbox-'.$metod.'-pay.lytex.com.br';
        }
	}

    /**
     * Criar o token de acesso a API Lytex Pagamentos
     * @return mixed
     */
    public function post_access_token() {
        $path = '/v1/oauth/obtain_token';
        $body =  array(
            "grantType"     => "clientCredentials",
            "clientId"      => $this->Client_ID,
            "clientSecret"  => $this->Client_Secret,
            "scopes"        =>[
                "client",
                "invoice",
                "paymentLink",
                "product"
            ]
        );
        $result =  $this->_api_service->post($this->get_base_url('auth'), $path, $body);
        return $result->data->accessToken;

    }

    /**
     * Remover caracteres não numéricos de uma string
     * @param $str
     * @return array|string|string[]|null
     */
    function limpar_texto($str){
        return preg_replace("/[^0-9]/", "", $str);
    }

    /**
     * Gerar uma fatura para o cliente
     * @param $post
     * @param $itens
     * @return mixed
     */
    public function post_create_invoice($post = null, $itens = null){
        $path = '/v1/invoices';
        $_cpf = $this->limpar_texto($post['lypay_cpf']);
        $_cellphone = $this->limpar_texto($post['lypay_phone']);
        $header= array(
            "Authorization" => "Bearer " . $this->post_access_token(),
            "recipientId"   => $this->recipientId,
            "Content-Type"  => "application/json"
        );
        $datetime = new DateTime();

        $body = array(

            "client"=> [
                "treatmentPronoun"=> "you",
                "name"=> $post['lypay_name'],
                "type"=> "pf",
                "cpfCnpj"=> $_cpf,
                "email"=> $post['lypay_email'],
                "cellphone"=> $_cellphone,
                "address"=> [
                    "number"=> "77",
                    "zip"=> str_replace('-','', $post['lypay_zip_code']),
                    "city"=> $post['lypay_city'],
                    "street"=> $post['lypay_address'],
                    "state"=> $post['lypay_state'],
                    "zone"=> $post['lypay_zone']
                ]
            ],
            "items"=> $itens,
            "dueDate"=> $datetime->format(DateTime::ATOM),

            "paymentMethods"=> [
              "pix"=> [
                "enable"=> $post['lypay_finalPaymentMethod'] == "pix" ? true: false
              ],
              "boleto"=> [
                "enable"=> $post['lypay_finalPaymentMethod'] == "billet" ? true: false,
                "dueDateDays"=> 2
              ],
              "creditCard"=> [
                "enable"=> false
              ]
            ]
        );
        $result =  $this->_api_service->post($this->get_base_url('api'), $path, $body, $header);
        return $result->data;
    }

    /**
     * validar os dados envido via hook da API
     * @param $data
     * @return void
     */
    public function validate_hook_data($data){
        $status = $data['webhookType'];
        if($status == "invoiceLiquidation"){
            $response = array(
                'id'=> $data['data']['referenceId'],
                'status'=> $status
            );
        }
        if($status == "cancelled"){
            $response = array(
                'id'=> $data['data']['referenceId'],
                'status'=> $status
            );
        };
        return $response;

    }

}

