<?php
//echo plugin_dir_path(dirname(__FILE__)) . 'templates/order-received.php';

/**
 * Este gancho de ação registra nossa classe PHP como um gateway de pagamento WooCommerce
 */
add_filter( 'woocommerce_payment_gateways', 'lytex_add_gateway_class' );
function lytex_add_gateway_class( $gateways ) {
	$gateways[] = 'WC_lytex_Gateway'; // o nome da sua classe está aqui
	return $gateways;
}

/*
 * A classe em si, observe que está dentro do gancho de ação plugins_loaded
 */
add_action( 'plugins_loaded', 'lytex_init_gateway_class' );
function lytex_init_gateway_class() {

	class WC_lytex_Gateway extends WC_Payment_Gateway {

		/**
		 *Construtor de classe
		 */
		public function __construct() {

			$this->lytex_include();

			$this->id = 'lytex'; // ID do plug-in do gateway de pagamento
			$this->icon = get_site_url(). '/wp-content/plugins/lytex-pagamentos/assets/img/logo-dark.png'; // URL do ícone que será exibido na página de checkout próximo ao nome do seu gateway
			$this->has_fields = true; // caso você precise de um formulário de cartão de crédito personalizado
			$this->method_title = 'Lytex Pagamentos';
			$this->method_description = 'Com a Lytex Pagamentos você pode receber pagamentos via Boleto Bancário, Cartão de Crédito!'; // será exibido na página de opções

			// gateways podem suportar assinaturas, reembolsos, métodos de pagamento salvos,
			$this->supports = array(
				'products'
			);

			// Método com todos os campos de opções
			$this->init_form_fields();
			$this->includes();

			// Carregue as configurações.
			$this->init_settings();

			$this->title 			= $this->get_option( 'title' );
			$this->description 		= $this->get_option( 'description' );
			$this->enabled 			= $this->get_option( 'enabled' );
			$this->sandbox 			= $this->get_option( 'sandbox' );



			$this->Client_ID 		= $this->get_option( 'Client_ID' );
			$this->Client_Secret 	= $this->get_option( 'Client_Secret' );
			$this->recipientId 		= $this->get_option('recipient_Id');
			//$this->Callback_Secret = $this->get_option( 'Callback_Secret' );
			$this->transparent_checkout = 'yes';//$this->get_option('transparent_checkout');


			$this->billet_option 	= $this->get_option('billet');

			//$this->credit_card_option = $this->get_option('credit_card');
			$this->pix_option 		= $this->get_option('pix');

			// This action hook saves the settings
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
			add_action('woocommerce_thankyou_lytex', array($this, 'thankyou_page_lytex'));
			// We need custom JavaScript to obtain a token
			add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );

			$this->_api_service = new ApiService_Lytex_Pagamentos();


            add_action('woocommerce_api_' . $this->id , array($this, 'webhook'));



		}

        /**
         * Recebe o retorno da API via hook quando for feita a  mudança de status na fatura, no sistema
         * @return void
         */
        public function webhook() {
            @ob_clean(); // phpcs:ignore Generic.PHP.NoSilencedErrors.Discouraged
            $api = new Lytex_pagamentos_API();
            $data = json_decode(file_get_contents('php://input'), true);
            $checks = $api->validate_hook_data( $data );
            if($checks){
                header( 'HTTP/1.1 200 OK' );
                $this->change_status($checks);
            }else{
                wp_die( esc_html__( 'Lytex Request Unauthorized' ), array( 'response' => 401 ) );
            }
        }

        /**
         * recebe as informaçòes do hook e muda o status do pedido de compra
         * @param $checks
         * @return void
         */
        private function change_status($checks){
            $order = wc_get_order($checks['id']);
            switch ($checks['status']){
                case "invoiceLiquidation":
                    $order->update_status('processing', __('Paid'));
                    $order->payment_complete();
                    break;
                case "cancelled":
                    $order->update_status('cancelled', __('cancelled'));
                    break;
            }

        }


        /**
         * Exibe Informações na pagina de thankyou page
         * @param $order_id
         * @return void
         */
		public function thankyou_page_lytex($order_id){
			echo wpautop(wptexturize($this->lytex_thankyou_page($order_id)));
		}

        /**
         * Monta a estrutura da pagina de thankyou page
         * @param $order_id
         * @return false|string
         */
		protected function lytex_thankyou_page($order_id){

			$this->lytex_include();
			$order = wc_get_order($order_id);
			ob_start();
			include_once dirname( __FILE__ ) . '/templates/order-received.php';
			$html = ob_get_clean();
			return $html;
		}

        /**
         * Faz toda a inclusão da API
         * @return void
         */
		public function includes(){
			include_once dirname( __FILE__ ) . '/api/class-api-lytex.php';
			include_once dirname( __FILE__ ) . '/api/api_service.php';
			include_once 'includes/libs/LytexValidate.php';
		}

        /**
         * Faz toda a inclusão de folhas de style e JS para as configurações do dashboard
         * @return void
         */
		public function lytex_include(){

			if(is_admin() == true):
				 if(isset($_GET['section'])):
					 if($_GET['section'] ==  "lytex"):
						 wp_enqueue_script( 'Lytexs_recaptcha', 'https://www.google.com/recaptcha/api.js?render='. $this->recaptcha_lytex_login());
						 wp_enqueue_script( 'lytexscript', plugins_url() . '/lytex-pagamentos/assets/js/admin/main.js');
						 wp_enqueue_style( 'lytex-adm-style', plugins_url(). '/lytex-pagamentos/assets/css/style-adm-lytex.css' );
						 $this->lytex_login_modal_adm();
					 endif;
				 endif;
			 endif;

			wp_enqueue_script( 'lytex-checkout-script', plugins_url() . '/lytex-pagamentos/assets/js/checkout/main.js');
			wp_enqueue_style( 'lytex-checkout-style', plugins_url(). '/lytex-pagamentos/assets/css/style-checkout-lytex.css' );

		}

        private function lytex_login_modal_adm(){
			   include plugin_dir_path(dirname(__FILE__)) . 'lytex-pagamentos/templates/views/modal-admin.php';
		}
		public function recaptcha_lytex_login(){
			   return "6LdwU5scAAAAAGHCYryak7ZGmonJxhpEUA_ezLIG";
		}


        /**
         * Cria os formulario no dashboard
         * @return void
         */
		public function init_form_fields(){
			$this->form_fields = array(
				'enabled' => array(
					'title'       => 'Enable/Disable',
					'label'       => 'Habilitar Lytex Pagamentos',
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'title' => array(
					'title'       => 'Title',
					'type'        => 'text',
					'description' => 'Titulo do metodo de pagamento checkout.',
					'default'     => 'Lytex Pagamentos',
					'desc_tip'    => true,
				),
				'description' => array(
					'title'       => 'Description',
					'type'        => 'textarea',
					'description' => 'Descrição da tela de checkout.',
					'default'     => 'Pague pelo cartão de credito através do Lytex Pagamentos',
				),
				/*'transparent_checkout' => array(
					'title'       => 'Checkout Transparente',
					'label'       => 'Habilitar o Checkout Transparente.',
					'type'        => 'checkbox',
					'default'     => 'no',
				),*/
				'sandbox' => array(
					'title'       => 'Sandbox',
					'label'       => 'Modo Sandbox',
					'type'        => 'checkbox',
					'description' => 'Habilitar o modo de testes do Lytex Pagamentos',
					'default'     => 'yes',
					'desc_tip'    => true,
				),

				'recipient_Id' => array(
					'title'       => 'ID do Estabelecimento',
					'type'        => 'text'
				),

				'Client_ID' => array(
					'title'       => 'Client ID',
					'type'        => 'text'
				),
				'Client_Secret' => array(
					'title'       => 'Client Secret',
					'type'        => 'text',
				),
				'billet' => array(
					'title'       => 'Boleto',
					'label'       => 'Habilitar Pagamento por Boleto',
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'credit_card' => array(
					'title'       => 'Cartão de Crédito',
					'label'       => 'EM BREVE',
					'type'        => 'checkbox',
					'description' => '',
					'disabled'    => true,
					'default'     => 'no'
				),
				'pix' => array(
					'title'       => 'Pix',
					'label'       => 'Habilitar Pagamento por Pix',
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				)
			);
		}

         /**
         * Você precisará dele se quiser seu formulário de cartão de crédito personalizado
         * @return void
         */
		public function payment_scripts() {
            wp_enqueue_script( 'lytex-checkout-script', plugins_url() . '/lytex-pagamentos/assets/js/checkout/main.js','' ,rand(1, 1000), true);

			// precisamos de JavaScript para processar um token apenas nas páginas de carrinho/checkout, certo?
			if ( ! is_cart() && ! is_checkout() && ! isset( $_GET['pay_for_order'] ) ) {
				return;
			}

			// se nosso gateway de pagamento estiver desabilitado, não precisamos enfileirar o JS também
			if ( 'no' === $this->enabled ) {
				return;
			}

			// não há motivo para enfileirar JavaScript se as chaves de API não estiverem definidas
			if ( empty( $this->private_key ) || empty( $this->publishable_key ) ) {
				return;
			}

			// não trabalhe com detalhes do cartão sem SSL, a menos que seu site esteja em modo de teste
			if ( ! $this->testmode && ! is_ssl() ) {
				return;
			}

			// vamos supor que é o JavaScript do nosso processador de pagamentos que permite obter um token
			wp_enqueue_script( 'lytex_js', 'https://www.lytexpayments.com/api/token.js' );

			// e este é o nosso JS personalizado em seu diretório de plugins que funciona com token.js
			wp_register_script( 'woocommerce_lytex', plugins_url( 'lytex.js', __FILE__ ), array( 'jquery', 'lytex_js' ) );

			// na maioria dos processadores de pagamento, você precisa usar a CHAVE PÚBLICA para obter um token
			wp_localize_script( 'woocommerce_lytex', 'lytex_params', array(
				'publishableKey' => $this->publishable_key
			) );

			wp_enqueue_script( 'first' );

		}

        /**
         * Validação de campos
         * @return void
         */
		public function validate_fields() {

			$validate = new LytexValidate();

			$validation_arr = array(
				'payment_method' => array(
					'validation' => $validate->_payment_method( $_POST['lypay_finalPaymentMethod'] ),
					'msg'        => __('The selected payment method does not exist.')
				),
				'name'           => array(
					'validation' => $validate->_name( $_POST['lypay_name'] ),
					'msg'        => __('Full name is required.')
				),
				'type'           => array(
					'validation' => $validate->_person_type( $_POST['lypay_type'] ),
					'msg'        => __('The selected person type does not exist.')
				),
				'cpf'            => array(
					'validation' => $validate->_pf_pj( array( $_POST['lypay_type'], $_POST['lypay_cpf'] ) ),
					'msg'        => __('Enter a valid CPF/CNPJ.')
				),
				'phone'          => array(
					'validation' => $validate->_phone( $_POST['lypay_phone'] ),
					'msg'        => __('Please enter a valid phone number.')
				),
				'email'          => array(
					'validation' => $validate->_email( $_POST['lypay_email'] ),
					'msg'        => __('Enter a valid email address.')
				),
				'zip_code'       => array(
					'validation' => $validate->_zip_code( $_POST['lypay_zip_code'] ),
					'msg'        => __('Enter an existing zip code.')
				),
				'state'          => array(
					'validation' => $validate->_state( $_POST['lypay_state'] ),
					'msg'        => __('Select an existing state.')
				),
				'adress'         => array(
					'validation' => $validate->_street( $_POST['lypay_address'] ),
					'msg'        => __('Enter the full address.')
				),
				'adress_number'  => array(
					'validation' => $validate->_number( $_POST['lypay_number'] ),
					'msg'        => __('Enter the address number.')
				),
				'city'           => array(
					'validation' => $validate->_city( $_POST['lypay_city'] ),
					'msg'        => __('Enter the city.')
				),
				'zone'           => array(
					'validation' => $validate->_zone( $_POST['lypay_zone'], $this->transparent_checkout ),
					'msg'        => __('Enter the neighborhood.')
				)
			);

			foreach ( $validation_arr as $key => $value ) {

				if ( ! $value['validation'] ) {
					wc_add_notice( $value['msg'], 'error' );
				}

			}

		}

        /**
         * Função retorna o formato do preço
         * @param $value
         * @return string
         */
        public function ly_price_format($value){
            $value = number_format($value, 2, "", "");
            return $value;
        }

        /**
         * Processando o pagamento do cliente
         * @param $order_id
         * @return string[]|void
         */
		public function process_payment( $order_id ) {
			$order = wc_get_order( $order_id );
			$products = array();
			foreach ( $order->get_items() as $item_id => $item ) {
               // print_r($item);
				$products[] = [
					'name' => $item->get_name(),
					'quantity' => $item->get_quantity(),
					'value' => (int)$this->ly_price_format($item['line_subtotal'] / $item['qty'])
				];
			}


			if($order->get_total_tax() > 0){

				$tax_items_labels = array();
				$shipping_tax_label = '';

				foreach ($order->get_items('tax') as $tax_item){

					$tax_items_labels[$tax_item->get_rate_id()] = $tax_item->get_label();

					if( !empty($tax_item->get_shipping_tax_total()) ){
						$shipping_tax_label = $tax_item->get_label();
					}

				}

				$wc_tax = new WC_Tax();

				foreach ( $order->get_items() as $item_id => $item ) {

					$taxes = $item->get_taxes();

					foreach ( $taxes['subtotal'] as $rate_id => $tax ){

						$tax_label = $tax_items_labels[$rate_id];

					}

					$products[] = [
						'name' => $item->get_name(),
						'quantity' => $item->get_quantity(),
						'value' => ($item->get_subtotal()/$item->get_quantity()) * 100,
					];

				}

				$products[] = array(
					'name' => $tax_label,
					'quantity' => 1,
					'value' => $order->get_total_tax() * 100
				);

			} else {

				foreach ( $order->get_items() as $item_id => $item ) {
					$products[] = [
						'name' => $item->get_name(),
						'quantity' => $item->get_quantity(),
						'value' => ($item->get_subtotal()/$item->get_quantity()) * 100,
					];
				}

			}

			if($order->get_shipping_total() > 0){

				$products[] = array(
					'name' => 'Frete',
					'quantity' => 1,
					'value' => $order->get_shipping_total() * 100
				);

			}

			$api = new Lytex_pagamentos_API();
			$result = $api->post_create_invoice($_POST, $products);

            print_r($result);

			if($this->transparent_checkout == 'yes'){
				//Verifica se ocorreu algum erro na API
				if($result->error->details[0]->message == true){
					wc_add_notice($result->error->details[0]->message, 'error');
				}else{
					//verifica se e boleto
					$PaymentMethodLytex =$_POST['lypay_finalPaymentMethod'];
					if($PaymentMethodLytex == "billet"){
						add_post_meta(intval($order_id), 'billet', $result->paymentMethods->boleto->barcode, true);
						update_post_meta(intval($order_id), '_payment_method_title', sanitize_text_field(__('Billet Banking - Lytex Pagamentos')));
						$order->update_status( 'on-hold', __( 'Awaiting offline payment', 'wc-gateway-offline' ) );
						return array(
							'result'   => 'success',
							'redirect' => $this->get_return_url( $order ) . '&PaymentMethod='.$_POST['lypay_finalPaymentMethod']
						);
						//verifica se e pix
					}elseif($PaymentMethodLytex == "pix"){
						add_post_meta(intval($order_id), 'billet', $result->paymentMethods->pix->qrcode, true);
						update_post_meta(intval($order_id), '_payment_method_title', sanitize_text_field(__('PIX - Lytex Pagamentos')));
						$order->update_status( 'on-hold', __( 'Awaiting offline payment', 'wc-gateway-offline' ) );
						return array(
							'result'   => 'success',
							'redirect' => $this->get_return_url( $order ) . '&PaymentMethod='.$_POST['lypay_finalPaymentMethod']
						);
						//retorna falso caso não seja nenhuma das opções
					}else{
						return array(
							'result'   => 'fail',
							'redirect' => '',
						);

					}
				}
			} else {

				$order->update_status( 'on-hold', __( 'Awaiting offline payment', 'wc-gateway-offline' ) );
				return array(
					'result'   => 'success',
					'redirect' => $api->get_base_url('checkout').'/fatura/'.$result->_id
				);

			}
		}

        /**
         * Criando os campos do formulario Lytex
         * @return void
         */
		public function payment_fields() {
			wp_register_script('jquery-wp', '/wp-includes/js/jquery/jquery.js', false);
			wp_enqueue_script('jquery-wp');
			wp_enqueue_script('jquery-mask', plugins_url('/lytex-pagamentos/assets/js/checkout/jquery.mask.js', plugin_dir_path(__FILE__)), array('jquery-wp'), '', true);
			wp_enqueue_script( 'lytex-checkout-script', plugins_url() . '/lytex-pagamentos/assets/js/checkout/main.js');

			// ok, let's display some description before the payment form
			if ( $this->description ) {
				// you can instructions for test mode, I mean test card numbers etc.
				if ( $this->sandbox == "yes" ) {
					$this->description = __('Sandbox mode is activated, invoices created must not be paid.');
					$this->description  = trim( $this->description );
				}
				// display the description with <p> tags etc.
				echo wpautop( wp_kses_post( $this->description ) );
			}


			$billet_option = $this->billet_option;
			//$credit_option = $this->credit_card_option;
			$pix_option = $this->pix_option;

			if($this->transparent_checkout == 'no'){
				ob_start();
				include plugin_dir_path(dirname(__FILE__)) . 'lytex-pagamentos/templates/lytex-hdn-checkout.php';
				$html = ob_get_clean();
				echo $html;
			} else {
				ob_start();
				include plugin_dir_path(dirname(__FILE__)) . 'lytex-pagamentos/templates/lytex-checkout.php';
				$html = ob_get_clean();
				echo $html;
			}

		}
 	}
}